@@ -34,11 +34,7 @@ struct fio_mutex *fio_mutex_init(int value)
 	}
 
 #ifdef FIO_HAVE_FALLOCATE
-	ret = posix_fallocate(fd, 0, sizeof(struct fio_mutex));
-	if (ret > 0) {
-		fprintf(stderr, "posix_fallocate mutex failed: %s\n", strerror(ret));
-		goto err;
-	}
+	posix_fallocate(fd, 0, sizeof(struct fio_mutex));
 #endif
 
 	if (ftruncate(fd, sizeof(struct fio_mutex)) < 0) {
@@ -93,6 +89,9 @@ struct fio_mutex *fio_mutex_init(int value)
 		goto err;
 	}
 
+	pthread_condattr_destroy(&cond);
+	pthread_mutexattr_destroy(&attr);
+
 	return mutex;
 err:
 	if (mutex)
